@@ -1,6 +1,6 @@
 /*
  * Rjb - Ruby <-> Java Bridge
- * Copyright(c) 2004,2005,2006,2007,2008,2009,2010,2011,2012 arton
+ * Copyright(c) 2004,2005,2006,2007,2008,2009,2010,2011,2012,2014 arton
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,10 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: rjb.c 199 2012-12-17 13:31:18Z arton $
  */
 
-#define RJB_VERSION "1.4.9"
+#define RJB_VERSION "1.5.3"
 
 #include "ruby.h"
 #include "extconf.h"
@@ -846,9 +845,13 @@ static void rv2jobject(JNIEnv* jenv, VALUE val, jvalue* jv, const char* psig, in
 #endif                
             case T_OBJECT:
             default:
-#if DEBUG
+#if defined(DEBUG)
+              {
+                VALUE v = rb_funcall(val, rb_intern("inspect"), 0);
                 fprintf(stderr, "rtype:%d, sig=%s\n", TYPE(val), psig);
+                fprintf(stderr, "obj:%s\n", StringValueCStr(v));
                 fflush(stderr);
+              }
 #endif
                 rb_raise(rb_eRuntimeError, "can't convert to java type");
                 break;
@@ -2127,12 +2130,19 @@ static VALUE register_instance(JNIEnv* jenv, VALUE klass, struct jv_data* org, j
     return v;
 }
 
+#define IS_BYTE(b) (!((b) & 0xffffff00))
+#define IS_SHORT(b) (!((b) & 0xffff0000))
 /*
  * temporary signature check
  * return !0 if found
  */
+#define UNMATCHED 0
+#define SATISFIED 1
+#define SOSO 2
+#define PREFERABLE 3
 static int check_rtype(JNIEnv* jenv, VALUE* pv, char* p)
 {
+    size_t i;
     char* pcls = NULL;
     if (*p == 'L')
     {
@@ -2146,52 +2156,89 @@ static int check_rtype(JNIEnv* jenv, VALUE* pv, char* p)
     }
     if (pcls && !strcmp("java.lang.Object", pcls))
     {
-        return 1;
+        return SATISFIED;
     }
     switch (TYPE(*pv))
     {
     case T_FIXNUM:
-        return strchr("BCDFIJS", *p) != NULL;
+        if (strchr("IJ", *p)) return SOSO;
+        return strchr("BCDFS", *p) != NULL;
     case T_BIGNUM:
         return strchr("BCDFIJS", *p) != NULL;        
     case T_FLOAT:
-	return strchr("DF", *p) != NULL;
+        if (*p == 'D') return SOSO;
+        if (*p == 'F') return SATISFIED;
+        return UNMATCHED;
     case T_STRING:
-        return pcls && !strcmp("java.lang.String", pcls) || *p == '[' && *(p + 1) == 'B';
+        if (pcls && (!strcmp("java.lang.String", pcls)
+                     || !strcmp("java.lang.CharSequence", pcls)))
+        {
+            return PREFERABLE;
+        }
+        else if (*p == '[' && *(p + 1) == 'B')
+        {
+            return SATISFIED;
+        }
+        return UNMATCHED;
     case T_TRUE:
     case T_FALSE:
-        return *p == 'Z';
+        return (*p == 'Z') ? SOSO : UNMATCHED;
     case T_ARRAY:
-        return *p == '[';
+        if (*p == '[')
+        {
+            int weight = (*(p + 1) == 'C') ? SOSO : PREFERABLE;
+            size_t len = RARRAY_LEN(*pv);
+            VALUE* ppv = RARRAY_PTR(*pv);
+            unsigned long ul;
+            if (!strchr("BCSI", *(p + 1))) return SOSO; // verify later
+            if (len > 32) len = 32;
+            for (i = 0; i < len; i++, ppv++)
+            {
+                if (!FIXNUM_P(*ppv))
+                {
+                    return UNMATCHED;
+                }
+                ul = (unsigned long)FIX2LONG(*ppv);
+                if (*(p + 1) == 'B')
+                {
+                    if (!IS_BYTE(ul)) return UNMATCHED;
+                }
+                else if (*(p + 1) == 'C' || *(p + 1) == 'S')
+                {
+                    if (!IS_SHORT(ul)) return UNMATCHED;
+                }
+            }
+            return weight;
+        }
+        return UNMATCHED;
     case T_DATA:
+    case T_OBJECT:
         if (IS_RJB_OBJECT(*pv) && pcls)
 	{
 	    /* imported object */
 	    jclass cls;
             struct jvi_data* ptr;
 	    int result = 0;
-            if (!strcmp("java.lang.String", pcls)) return 1;
+            if (!strcmp("java.lang.String", pcls)) return SATISFIED;
 	    Data_Get_Struct(*pv, struct jvi_data, ptr);
             RJB_FIND_CLASS(cls, java2jniname(pcls));
 	    if (cls)
-        	    {
+            {
 	        result = (cls && (*jenv)->IsInstanceOf(jenv, ptr->obj, cls));
 	        (*jenv)->DeleteLocalRef(jenv, cls);
 	    }
-	    return result;
+	    return (result) ? PREFERABLE : UNMATCHED;
 	} else if (pcls) {
             VALUE blockobj = rb_class_new_instance(1, pv, rjba);
             *pv = rjb_s_bind(rjbb, blockobj, rb_str_new2(pcls));
         }
 	/* fall down to the next case */
-    case T_OBJECT:
-	/* fall down to the next case */
     default:
         if (pcls || *p == '[')
         {
-            return 1;
+            return SATISFIED;
         }
-	return 0;
+	return UNMATCHED;
     }
 }
 
@@ -2235,7 +2282,10 @@ static VALUE rjb_newinstance(int argc, VALUE* argv, VALUE self)
     VALUE ret = Qnil;
     struct jv_data* ptr;
     struct cls_constructor** pc;
+    struct cls_constructor** found_pc = NULL;
     int found = 0;
+    int weight = 0;
+    int cweight;
     JNIEnv* jenv = rjb_prelude();
     
     Data_Get_Struct(self, struct jv_data, ptr);
@@ -2246,28 +2296,43 @@ static VALUE rjb_newinstance(int argc, VALUE* argv, VALUE self)
 	char* psig;
 	for (pc = ptr->constructors; *pc; pc++)
 	{
+            found = 0;
 	    if ((*pc)->arg_count == argc)
 	    {
-	        found = 1;
+                found = 1;
+                cweight = 0;
 		psig = (*pc)->method_signature;
 		for (i = 0; i < argc; i++)
 		{
-		    if (!check_rtype(jenv, argv + i, psig))
+                    int w = check_rtype(jenv, argv + i, psig);
+		    if (!w)
                     {
 		        found = 0;
 			break;
 		    }
+                    cweight += w;
 		    psig = next_sig(psig);
 		}
-		if (found)
-		{
-		    ret = createinstance(jenv, argc, argv, self, *pc);
-		    break;
-		}
 	    }
+            if (found)
+            {
+                if (cweight > weight || weight == 0)
+                {
+                    found_pc = pc;
+                    weight = cweight;
+                }
+            }
 	}
+        if (found_pc)
+        {
+#if defined(DEBUG)
+            fprintf(stderr, "ctr sig=%s\n", (*found_pc)->method_signature);
+            fflush(stderr);
+#endif
+            ret = createinstance(jenv, argc, argv, self, *found_pc);
+        }
     }
-    if (!found) {
+    if (!found_pc) {
 	rb_raise(rb_eRuntimeError, "Constructor not found");
     }
     return ret;
@@ -2364,7 +2429,8 @@ static VALUE rjb_get_ctor_signatures(VALUE self)
     {
         for (pc = ptr->constructors; *pc; pc++)
         {
-            rb_ary_push(ret, rb_str_new2((*pc)->method_signature));
+            const char* sig = (*pc)->method_signature;
+            rb_ary_push(ret, rb_str_new2(sig ? sig : ""));
         }
     }
     return ret;
@@ -2861,11 +2927,13 @@ static void setter(JNIEnv* jenv, struct cls_field* pf,  struct jvi_data* ptr, VA
 static VALUE invoke(JNIEnv* jenv, struct cls_method* pm, struct jvi_data* ptr,
 		    int argc, VALUE* argv, const char* sig)
 {
-    int i, found;
+    int i, found, cweight;
     jvalue jv;
     jvalue* args;
     char* psig;
     struct cls_method* orgpm = pm;
+    struct cls_method* prefer_pm = NULL;
+    int weight = 0;
 
     if (rb_block_given_p())
     {
@@ -2876,8 +2944,9 @@ static VALUE invoke(JNIEnv* jenv, struct cls_method* pm, struct jvi_data* ptr,
         argv = pargs;
     }
 
-    for (found = 0; pm; pm = pm->next)
+    for (; pm; pm = pm->next)
     {
+        found = 0;
         if (argc == pm->basic.arg_count)
         {
             if (sig && pm->basic.method_signature) 
@@ -2885,27 +2954,38 @@ static VALUE invoke(JNIEnv* jenv, struct cls_method* pm, struct jvi_data* ptr,
                 if (!strcmp(sig, pm->basic.method_signature))
                 {
 	            found = 1;
+                    prefer_pm = pm;
                     break;
 		}
 	    }
             else
             {
-	        psig = pm->basic.method_signature;
                 found = 1;
+                cweight = 0;
+	        psig = pm->basic.method_signature;
                 for (i = 0; i < argc; i++)
                 {
-		    if (!check_rtype(jenv, argv + i, psig))
+                    int w = check_rtype(jenv, argv + i, psig);
+		    if (!w)
                     {
                         found = 0;
                         break;
                     }
+                    cweight += w;
 		    psig = next_sig(psig);
                 }
-                if (found) break;
 	    }
 	}
+        if (found)
+        {
+            if (cweight > weight || weight == 0)
+            {
+                prefer_pm = pm;
+                weight = cweight;
+            }
+        }
     }
-    if (!found)
+    if (!prefer_pm)
     {
 	const char* tname = rb_id2name(orgpm->name);
 	if (sig) 
@@ -2918,71 +2998,71 @@ static VALUE invoke(JNIEnv* jenv, struct cls_method* pm, struct jvi_data* ptr,
 	}
     }
     args = (argc) ? ALLOCA_N(jvalue, argc) : NULL;
-    psig = pm->basic.method_signature;
+    psig = prefer_pm->basic.method_signature;
     for (i = 0; i < argc; i++)
     {
-	R2J pr2j = *(pm->basic.arg_convert + i);
+	R2J pr2j = *(prefer_pm->basic.arg_convert + i);
 	pr2j(jenv, argv[i], args + i, psig, 0);
 	psig = next_sig(psig);
     }
-    switch (pm->basic.result_signature)
+    switch (prefer_pm->basic.result_signature)
     {
     case 'D':
       {
-        INVOKEAD voked = *(INVOKEAD*)(((char*)*jenv) + pm->method);
-	jv.d = voked(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEAD voked = *(INVOKEAD*)(((char*)*jenv) + prefer_pm->method);
+	jv.d = voked(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
     case 'Z':
     case 'B':
       {
-        INVOKEAZ vokez = *(INVOKEAZ*)(((char*)*jenv) + pm->method);
-	jv.z = vokez(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEAZ vokez = *(INVOKEAZ*)(((char*)*jenv) + prefer_pm->method);
+	jv.z = vokez(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
     case 'F':
       {
-        INVOKEAF vokef = *(INVOKEAF*)(((char*)*jenv) + pm->method);
-	jv.f = vokef(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEAF vokef = *(INVOKEAF*)(((char*)*jenv) + prefer_pm->method);
+	jv.f = vokef(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
     case 'C':
     case 'S':
       {
-        INVOKEAS vokes = *(INVOKEAS*)(((char*)*jenv) + pm->method);
-	jv.s = vokes(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEAS vokes = *(INVOKEAS*)(((char*)*jenv) + prefer_pm->method);
+	jv.s = vokes(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
 #if HAVE_LONG_LONG
     case 'J':
       {
-        INVOKEAL vokel = *(INVOKEAL*)(((char*)*jenv) + pm->method);
-	jv.j = vokel(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEAL vokel = *(INVOKEAL*)(((char*)*jenv) + prefer_pm->method);
+	jv.j = vokel(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
 #endif
     default:
       {
-        INVOKEA voke = *(INVOKEA*)(((char*)*jenv) + pm->method);
-        jv.l = voke(jenv, ptr->obj, pm->basic.id, args);
+        INVOKEA voke = *(INVOKEA*)(((char*)*jenv) + prefer_pm->method);
+        jv.l = voke(jenv, ptr->obj, prefer_pm->basic.id, args);
       }
       break;
     }
     rjb_check_exception(jenv, 1);
-    psig = pm->basic.method_signature;
+    psig = prefer_pm->basic.method_signature;
     for (i = 0; i < argc; i++)
     {
-	R2J pr2j = *(pm->basic.arg_convert + i);
+	R2J pr2j = *(prefer_pm->basic.arg_convert + i);
 	pr2j(jenv, argv[i], args + i, psig, 1);
 	psig = next_sig(psig);
     }
-    if (pm->basic.result_arraydepth)
+    if (prefer_pm->basic.result_arraydepth)
     {
-        return ja2r(pm->result_convert, jenv, jv, pm->basic.result_arraydepth);
+        return ja2r(prefer_pm->result_convert, jenv, jv, prefer_pm->basic.result_arraydepth);
     }
     else
     {
-        return pm->result_convert(jenv, jv);
+        return prefer_pm->result_convert(jenv, jv);
     }
 }
 
