@@ -147,25 +147,25 @@ class TestRjb < Test::Unit::TestCase
     if Object::const_defined?(:Encoding)
       test = import('jp.co.infoseek.hp.arton.rjb.Test').new
       euc_kj = "\xb4\xc1\xbb\xfa\xa5\xc6\xa5\xad\xa5\xb9\xa5\xc8".force_encoding Encoding::EUC_JP
-      s = @jString.new(euc_kj)
+      s = @jString.new_with_sig('Ljava.lang.String;', euc_kj)
       assert_equal(s.toString().encoding, Encoding::UTF_8)
       assert(test.isSameString(s))
       assert(test.isSameString(euc_kj))
       assert_equal(s.toString().encode(Encoding::EUC_JP), euc_kj)
       sjis_kj = "\x8a\xbf\x8e\x9a\x83\x65\x83\x4c\x83\x58\x83\x67".force_encoding Encoding::SHIFT_JIS
-      s = @jString.new(sjis_kj)
+      s = @jString.new_with_sig('Ljava.lang.String;', sjis_kj)
       assert_equal(s.toString().encoding, Encoding::UTF_8)
       assert(test.isSameString(s))
       assert(test.isSameString(sjis_kj))
       assert_equal(s.toString().encode(Encoding::SHIFT_JIS), sjis_kj)
       utf8_kj = "\xE6\xBC\xA2\xE5\xAD\x97\xE3\x83\x86\xE3\x82\xAD\xE3\x82\xB9\xE3\x83\x88".force_encoding Encoding::UTF_8
-      s = @jString.new(utf8_kj)
+      s = @jString.new_with_sig('Ljava.lang.String;', utf8_kj)
       assert_equal(s.toString().encoding, Encoding::UTF_8)
       assert(test.isSameString(s))
       assert(test.isSameString(utf8_kj))
       assert_equal(s.toString().encode(Encoding::UTF_8), utf8_kj)
       iso2022jp_kj = "\x1b\x24\x42\x34\x41\x3b\x7a\x25\x46\x25\x2d\x25\x39\x25\x48\x1b\x28\x42".force_encoding Encoding::ISO_2022_JP
-      s = @jString.new(iso2022jp_kj)
+      s = @jString.new_with_sig('Ljava.lang.String;', iso2022jp_kj)
       assert_equal(s.toString().encoding, Encoding::UTF_8)
       assert(test.isSameString(s))
       assert(test.isSameString(iso2022jp_kj))
@@ -802,13 +802,12 @@ class TestRjb < Test::Unit::TestCase
   end
 
   def test_reraise_exception()
-   unless /^1\.8/ =~ RUBY_VERSION
-      begin
-        cause_exception
-      rescue
-        assert($!.to_s =~ /NumberFormatException/)
-      end
-   end
+    skip('1.8 does not support reraise') if /^1\.8/ =~ RUBY_VERSION
+    begin
+      cause_exception
+    rescue
+      assert($!.inspect =~ /NumberFormatException/)
+    end
   end
 
 
@@ -874,10 +873,11 @@ class TestRjb < Test::Unit::TestCase
     assert_equal(cons._classname, sys._invoke('console')._classname)    
   end
   def test_longarg
+    skip('rbx can handle 64bits long') if RUBY_ENGINE == 'rbx'
     assert_equal(597899502607411822, @jLong.reverse(0x7654321076543210))
     begin
       @jLong.reverse(0x76543210765432101)
-      fail 'no exception for gibnum it doesn\'t convert Java long'
+      fail 'no exception for bigbnum it doesn\'t convert Java long'
     rescue RangeError
       assert true
     end
@@ -899,5 +899,29 @@ class TestRjb < Test::Unit::TestCase
     assert_equal '[Ljava.lang.Integer;', ret[1]
     assert_equal '[Ljava.net.URI;', ret[2]    
   end
+
+  SJIS_STR = "\x8a\xbf\x8e\x9a\x83\x65\x83\x4c\x83\x58\x83\x67"
+  EUCJP_STR = "\xb4\xc1\xbb\xfa\xa5\xc6\xa5\xad\xa5\xb9\xa5\xc8"
+  UTF8_STR = "\xE6\xBC\xA2\xE5\xAD\x97\xE3\x83\x86\xE3\x82\xAD\xE3\x82\xB9\xE3\x83\x88"
+  def test_auto_constructor_selection
+    skip 'no encoding' unless Object::const_defined?(:Encoding)
+    sys = import('java.lang.System')
+    encoding = sys.property('file.encoding')
+    s = @jString.new(SJIS_STR.force_encoding Encoding::SHIFT_JIS)
+    e = @jString.new(EUCJP_STR.force_encoding Encoding::EUC_JP)
+    u = @jString.new(UTF8_STR.force_encoding Encoding::UTF_8)
+    if encoding == 'MS932'
+      s1 = @jString.new(SJIS_STR.bytes)
+    elsif encoding.upcase == 'EUC-JP'
+      s1 = @jString.new(EUCJP_STR.bytes)
+    elsif encoding.upcase == 'UTF-8'
+      s1 = @jString.new(UTF8_STR.bytes)
+    else
+      skip 'no checkable encoding'
+    end
+    assert_equal s1.toString, s.toString
+    assert_equal s1.toString, e.toString
+    assert_equal s1.toString, u.toString
+  end
 end
 
